#include "stdafx.h"

//-- Global data 
char * g_szTitle = "GEM Data Configuration";

//-----------------------------------------------------------------------------
class CVwDataConfig : public CDialog { 
	private : 
		int m_iChannels;
		bool m_bDefault;

		//-- temp storage arrays while dialog is active for color arrays 
		COLORREF m_acrTraceColor[MAX_CHANNELS];		// Color select - local color array 
		COLORREF m_acrDigoutColor[DIGOUTLINES];		// Color select - local color array 
		HBRUSH hbrButton;

		//-- Controls 
		CDlgEdit m_edtTraces;
		CDlgEdit m_edtPoints;
		CDlgCheck m_chkOverlap;
		CDlgCheck m_chkDispLists;
		CDlgCheck m_chkDispListLabels;
		CDlgCheck m_chkDispCursors;
		CDlgCheck m_chkDispFitCurves;
		CDlgCheck m_chkDispIdealizations;
		CDlgCheck m_chkDispDigout;
		
		CDlgEdit m_aedtUnits[MAX_CHANNELS];
		CDlgEdit m_aedtName[MAX_CHANNELS];
		CDlgCheck m_achkVisible[MAX_CHANNELS];
		CDlgCheck m_achkBaseline[MAX_CHANNELS];
		CDlgCheck m_achkFilter[MAX_CHANNELS];
		CDlgEdit m_aedtFilterKhz[MAX_CHANNELS];
		CDlgCheck m_achkAutoScale[MAX_CHANNELS];
		CDlgButton m_btnColor[MAX_CHANNELS];
		
		CDlgButton m_btnDigoutColor[DIGOUTLINES];

	public : 		
		//-- Where to store results when done
		CDataConfigure * m_pConfig;

		CVwDataConfig( bool bDefault ) { 
			m_bDefault=bDefault;
			m_iID = IDD_DATAVIEWCONFIG;
			m_pConfig = NULL;
			hbrButton = NULL;
			}
	
		~CVwDataConfig() { 
			if( hbrButton != NULL ) 
				DeleteObject(hbrButton);
			}

		//-- Initialize controls 
		void InitializeControls( ) { 
			for( int i=0; i<MAX_CHANNELS; ++i ) 
				m_acrTraceColor[i]=RGB(0,0,16*i);
			for( int i=0; i<DIGOUTLINES; ++i ) 
				m_acrDigoutColor[i]=RGB(0,16*i,0);
			m_edtTraces.SetHandle( GetDlgItem(m_hDlg, IDC_EDTTRACES), "Traces per page");
			m_edtPoints.SetHandle( GetDlgItem(m_hDlg, IDC_EDTPOINTS), "Points per trace");
			m_chkOverlap.SetHandle( GetDlgItem(m_hDlg, IDC_CHKOVERLAP), "Overlap multiple channels within trace for larger resolution");
			
			m_chkDispLists.SetHandle( GetDlgItem(m_hDlg, IDC_DISPLISTS), "Toggle display of lists");
			m_chkDispListLabels.SetHandle( GetDlgItem(m_hDlg, IDC_DISPLISTLABELS), "Toggle display of list labels");
			m_chkDispCursors.SetHandle( GetDlgItem(m_hDlg, IDC_DISPCURSORS), "Toggle display of cursors");
			m_chkDispFitCurves.SetHandle( GetDlgItem(m_hDlg, IDC_DISPFITCURVES), "Toggle display of fit curves");
			m_chkDispIdealizations.SetHandle( GetDlgItem(m_hDlg, IDC_DISPIDEALIZATIONS), "Toggle display of idealizations");
			m_chkDispDigout.SetHandle( GetDlgItem(m_hDlg, IDC_DISPDIGOUT), "Toggle display of digital output");
			
			for( int i=0; i<MAX_CHANNELS; ++i ) { 
				m_aedtUnits[i].SetHandle( GetDlgItem(m_hDlg, IDC_EDTUNITS0+i), "Signal units ( Volt, mAmp, etc.)");
				m_aedtUnits[i].SetTextLimit( 64 );	//TODO - what should limit be ? (and use a constant!)
				m_aedtName[i].SetHandle( GetDlgItem(m_hDlg, IDC_EDTNAME0+i), "Channel Name");
				m_aedtName[i].SetTextLimit( 64 );	
				m_achkVisible[i].SetHandle( GetDlgItem(m_hDlg, IDC_CHKVISIBLE0+i), "Whether to display this channel");
				m_achkBaseline[i].SetHandle( GetDlgItem(m_hDlg, IDC_CHKBASELINE0+i), "Whether to draw the baseline");
				m_achkFilter[i].SetHandle( GetDlgItem(m_hDlg, IDC_CHKFILTER0+i), "Whether to filter the data");
				m_aedtFilterKhz[i].SetHandle( GetDlgItem(m_hDlg, IDC_EDTFILTERKHZ0+i), "If filtering is selected, level to filter at");
				m_achkAutoScale[i].SetHandle( GetDlgItem(m_hDlg, IDC_CHKAUTOSCALE0+i), "Whether to automatically scale this channel to fill the display range");
				m_btnColor[i].SetHandle( GetDlgItem(m_hDlg, IDC_BTNCOLOR0+i), "Change the display color for this channel");
				}
			char buf[128];
			for( int i=0; i<DIGOUTLINES; ++i ) {
				sprintf( buf, "Change display color for digital output %d", i );
				m_btnDigoutColor[i].SetHandle( GetDlgItem(m_hDlg, IDC_BTNDIGOUTCOLOR0+i), buf );
				}
			}

		//-- Set all control contents and disable unused rows
		void SetControlValues( ) { 
			m_iChannels = min( MAX_CHANNELS, m_pConfig->ChannelCount() );
			
			m_edtTraces.SetTextInt( m_pConfig->m_iTracesPerPage );
			m_edtPoints.SetTextInt( m_pConfig->m_iPointsPerTrace );
			m_chkOverlap.SetCheck( m_pConfig->m_bOverlap );

			m_chkDispLists.SetCheck( m_pConfig->m_bDispLists );
			m_chkDispListLabels.SetCheck( m_pConfig->m_bDispListLabels );
			m_chkDispCursors.SetCheck( m_pConfig->m_bDispCursors );
			m_chkDispFitCurves.SetCheck( m_pConfig->m_bDispFitCurves );
			m_chkDispIdealizations.SetCheck( m_pConfig->m_bDispIdealizations );
			m_chkDispDigout.SetCheck( m_pConfig->m_bDispDigout );
		
			CChannel * pChannel;
			for( int i=0; i<m_iChannels; ++i ) { 
				pChannel = (CChannel *)m_pConfig->Channel(i);
				if( ! m_bDefault ) {
					m_aedtUnits[i].SetText( pChannel->m_cUnits.Buffer() );
					m_aedtName[i].SetText( pChannel->m_cName.Buffer() );
					}
				m_achkVisible[i].SetCheck( pChannel->m_bVisible );
				m_achkBaseline[i].SetCheck( pChannel->m_bBaseline );
				m_achkFilter[i].SetCheck( pChannel->m_bFilter );
				m_aedtFilterKhz[i].SetTextDouble( pChannel->m_dFilterKhz );
				m_achkAutoScale[i].SetCheck( pChannel->m_bAutoScale );
				m_acrTraceColor[i]=pChannel->m_cr;
				}
				
			for( int i=0; i<MAX_CHANNELS; ++i ) { 
				if( m_bDefault ) { 
					m_aedtUnits[i].SetVisible( false );
					m_aedtName[i].SetVisible( false );
					}
				else { 
					bool bEnable = (i<m_iChannels);
					m_aedtUnits[i].SetEnabled( bEnable );
					m_aedtName[i].SetEnabled( bEnable );
					m_achkVisible[i].SetEnabled( bEnable );
					m_achkBaseline[i].SetEnabled( bEnable );
					m_achkFilter[i].SetEnabled( bEnable );
					m_aedtFilterKhz[i].SetEnabled( bEnable );
					m_achkAutoScale[i].SetEnabled( bEnable );
					m_btnColor[i].SetVisible( bEnable );
					}
				}

			for( int i=0; i<DIGOUTLINES; ++i )
				m_acrDigoutColor[i]=m_pConfig->m_acrDigout[i];
			}

		//-----------------------------------------------------------------------------
		void GetControlValues(  ) { 
			m_pConfig->m_iTracesPerPage = m_edtTraces.GetTextInt();
			m_pConfig->m_iPointsPerTrace = m_edtPoints.GetTextInt();
			m_pConfig->m_bOverlap = m_chkOverlap.GetCheck();
			m_pConfig->m_bDispLists = m_chkDispLists.GetCheck();
			m_pConfig->m_bDispListLabels = m_chkDispListLabels.GetCheck();
			m_pConfig->m_bDispCursors = m_chkDispCursors.GetCheck();
			m_pConfig->m_bDispFitCurves = m_chkDispFitCurves.GetCheck();
			m_pConfig->m_bDispIdealizations = m_chkDispIdealizations.GetCheck();
			m_pConfig->m_bDispDigout = m_chkDispDigout.GetCheck();
		
			CChannel * pChannel;
			for( int i=0; i<m_iChannels; ++i ) { 
				pChannel = (CChannel *)m_pConfig->Channel(i);
				if( ! m_bDefault ) {
					m_aedtUnits[i].GetText( &pChannel->m_cUnits );
					m_aedtName[i].GetText( &pChannel->m_cName );
					}
				pChannel->m_bVisible = m_achkVisible[i].GetCheck( );
				pChannel->m_bBaseline = m_achkBaseline[i].GetCheck( );
				pChannel->m_bFilter = m_achkFilter[i].GetCheck( );
				pChannel->m_dFilterKhz = m_aedtFilterKhz[i].GetTextDouble( );
				pChannel->m_bAutoScale = m_achkAutoScale[i].GetCheck( );
				pChannel->m_cr = m_acrTraceColor[i];
				}
			
			for( int i=0; i<DIGOUTLINES; ++i )
				m_pConfig->m_acrDigout[i] = m_acrDigoutColor[i];
			}

		//-----------------------------------------------------------------------------
		// Events and messages 

		//-- [CANCEL]  
		void OnBtnCancel( ) { 
			EndDialog( m_hDlg, IDCANCEL );
			}
			
		//-- [SAVE] 
		void OnSave( ) { 
			GetControlValues();
			EndDialog( m_hDlg, IDOK );
			}
			
		//-- [Color] selection button
		void OnBtnColorSelect( int iChannel, bool bDigout ) { 
			COLORREF acrTemp[16];
			memcpy( acrTemp, m_acrTraceColor, 16*sizeof(COLORREF) );
			
			CHOOSECOLOR cc;

			ZeroMemory(&cc, sizeof(cc));
			cc.lStructSize = sizeof(cc);
			cc.hwndOwner = m_hDlg;
			cc.lpCustColors = acrTemp; //NULL;	
			cc.rgbResult = (bDigout ? m_acrDigoutColor : m_acrTraceColor)[iChannel];
			cc.Flags = CC_FULLOPEN | CC_RGBINIT;	// CC_FULLOPEN CC_ANYCOLOR | 

			if( ChooseColor(&cc) ) 
				(bDigout ? m_acrDigoutColor : m_acrTraceColor)[iChannel]=cc.rgbResult;
			InvalidateRect( m_hDlg, NULL, false );
			}
			
		//-- For painting the color selection buttons, create a custom brush
		HBRUSH CreateButtonBrush( int iCh ) { 
			if( hbrButton != NULL ) 
				DeleteObject(hbrButton);
			return hbrButton = CreateSolidBrush( m_acrTraceColor[iCh] );
			}

		HBRUSH CreateDigoutButtonBrush( int iCh ) { 
			if( hbrButton != NULL ) 
				DeleteObject(hbrButton);
			return hbrButton = CreateSolidBrush( m_acrDigoutColor[iCh] );
			}
				
	};
	
CVwDataConfig * g_pVwDataConfig;		// Modal - one and only instance

bool OpenDataViewConfig( HWND hDlg, CDataConfigure * pConfig ) { 
	g_pVwDataConfig = new CVwDataConfig( false );
	g_pVwDataConfig->m_pConfig=pConfig;
	return ( DialogBox( GetInstance(), (LPCTSTR)IDD_DATAVIEWCONFIG, hDlg, (DLGPROC)DataConfigDlgProc ) == IDOK );
	}

bool OpenDataDefaultConfig( HWND hDlg ) {
	//-- Local copy may be unneeded.  Could access global object directly, 
	// but need to ensure [cancel] of dialogue will leave original unchanged.
	CDataConfigure oConfig;	
	oConfig.LoadDefaultDataConfig();
	
	while( oConfig.ChannelCount()<MAX_CHANNELS ) 
		oConfig.AddChannel();
	
	g_pVwDataConfig = new CVwDataConfig( true );
	g_pVwDataConfig->m_pConfig=&oConfig;
	if( DialogBox( GetInstance(), (LPCTSTR)IDD_DATAVIEWCONFIG, hDlg, (DLGPROC)DataConfigDlgProc ) == IDOK ) { 
		// save to registry and update global entry 
		oConfig.SetDefaultDataConfig();
		return true;
		}
	return false;
	}

//-----------------------------------------------------------------------------
// Message Pump
BOOL CALLBACK DataConfigDlgProc( HWND hDlg, UINT iMsg, WPARAM wParam, LPARAM lParam) { 
	int wmId, wmEvent;
	
	switch( iMsg ) { 
		//----- DIALOG INIT -  Control Initialization
		case WM_INITDIALOG :		
			g_pVwDataConfig->m_hDlg=hDlg;			
			
			//-- Clear Existing  Values
			g_pVwDataConfig->InitializeControls();
			g_pVwDataConfig->SetControlValues();
			return TRUE;

		//----- MENU and CONTROL Commands
		case WM_COMMAND : 
			wmId    = LOWORD(wParam); 
			wmEvent = HIWORD(wParam); 

			//if( wmId >= IDC_EDITUNITS0    && wmId < MAX_CHANNELS+IDC_EDITUNITS0     ) 
			//if( wmId >= IDC_EDTNAME0      && wmId < MAX_CHANNELS+IDC_EDTNAME0       ) 
			//if( wmId >= IDC_CHKVISIBLE0   && wmId < MAX_CHANNELS+IDC_CHKVISIBLE0    ) 
			//if( wmId >= IDC_CHKBASELINE0  && wmId < MAX_CHANNELS+IDC_CHKBASELINE0   ) 
			//if( wmId >= IDC_CHKFILTER0    && wmId < MAX_CHANNELS+IDC_CHKFILTER0     ) 
			//if( wmId >= IDC_EDTFILTERKHZ0 && wmId < MAX_CHANNELS+IDC_EDTFILTERKHZ0  ) 
			//if( wmId >= IDC_CHKAUTOSCALE0 && wmId < MAX_CHANNELS+IDC_CHKAUTOSCALE0  ) 
			if( wmId >= IDC_BTNCOLOR0 && wmId < MAX_CHANNELS+IDC_BTNCOLOR0 ) 
				g_pVwDataConfig->OnBtnColorSelect( wmId-IDC_BTNCOLOR0, false );
			if( wmId >= IDC_BTNDIGOUTCOLOR0 && wmId < DIGOUTLINES+IDC_BTNDIGOUTCOLOR0 ) 
				g_pVwDataConfig->OnBtnColorSelect( wmId-IDC_BTNDIGOUTCOLOR0, true );
				
			switch( wmId ) { 
				case IDOK :					//-- [OK]
					g_pVwDataConfig->OnSave( );
					break;

				case IDCANCEL :				//-- [CANCEL]
					g_pVwDataConfig->OnBtnCancel( );
					break;

				}
				
			return TRUE;

		case WM_CTLCOLORBTN :
			wmId = GetDlgCtrlID( (HWND)lParam );
			if( wmId >= IDC_BTNCOLOR0 && wmId < MAX_CHANNELS+IDC_BTNCOLOR0 )
				return (BOOL) g_pVwDataConfig->CreateButtonBrush(wmId-IDC_BTNCOLOR0);
			if( wmId >= IDC_BTNDIGOUTCOLOR0 && wmId < DIGOUTLINES+IDC_BTNDIGOUTCOLOR0 )
				return (BOOL) g_pVwDataConfig->CreateDigoutButtonBrush(wmId-IDC_BTNDIGOUTCOLOR0);
			return FALSE;

		case WM_PAINT : 
			/*{
			PAINTSTRUCT ps;
			HDC hdc = BeginPaint(hDlg, &ps);
							
			//----- Manual Drawing 
				
			//----- Done
			EndPaint(hDlg, &ps);
			}
			*/
			return false;

		case WM_DESTROY : 
			delete g_pVwDataConfig;
			WindowListRemove( hDlg );
			return TRUE;
			
		default : 
			break;
		}
	return FALSE;
	} 

